/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void IntTransmit( void );
void IntReceive( void );
void IntErr( uint8_t bLSErrType );
uint32_t rbSend( UARTn_Type* UARTx, uint8_t* txbuf, uint8_t buflen );
uint32_t rbReceive( UARTn_Type* UARTx, uint8_t* rxbuf, uint8_t buflen );
void UART1_IRQHandler_IT( void );
void UART1_Configure( void );
void UARTn_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// ring buffer size
#define RING_BUF_SIZE      32

//******************************************************************************
// Type
//******************************************************************************

typedef struct
{
   uint32_t    tx_head;                /**<  Tx ring buffer head index */
   uint32_t    tx_tail;                /**<  Tx ring buffer tail index */
   uint32_t    rx_head;                /**<  Rx ring buffer head index */
   uint32_t    rx_tail;                /**<  Rx ring buffer tail index */
   uint8_t     tx[RING_BUF_SIZE];      /**<  Tx data ring buffer */
   uint8_t     rx[RING_BUF_SIZE];      /**<  Rx data ring buffer */
} RING_BUF;

//******************************************************************************
// Macro
//******************************************************************************

// Buf mask
#define __BUF_MASK                        ( RING_BUF_SIZE - 1 )

// Check buf is full or not
#define __BUF_IS_FULL( head, tail )       ( (tail & __BUF_MASK) == ((head+1) & __BUF_MASK) )

// Check buf will be full in next receiving or not
#define __BUF_WILL_FULL( head, tail )     ( (tail & __BUF_MASK) == ((head+2) & __BUF_MASK) )

// Check buf is empty
#define __BUF_IS_EMPTY( head, tail )      ( (head & __BUF_MASK) == (tail & __BUF_MASK) )

// Reset buf
#define __BUF_INCR( bufidx )              ( bufidx = (bufidx+1) & __BUF_MASK )

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

// ring buffer
RING_BUF                rb;

// Current Tx Interrupt enable state
volatile FlagStatus     TxIntStat;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UART transmit function (ring buffer used)
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void IntTransmit( void )
{
   // Disable THRE interrupt
   HAL_UART_ConfigInterrupt( ( UARTn_Type* )UART1, UARTn_INTCFG_THRE, DISABLE );

   /* Wait until THR empty */
   while( HAL_UART_CheckBusy( ( UARTn_Type* )UART1 ) == SET );

   while( !__BUF_IS_EMPTY( rb.tx_head, rb.tx_tail ) )
   {
      /* Move a piece of data into the transmit FIFO */
      if( HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )&rb.tx[rb.tx_tail], 1, NONE_BLOCKING ) )
      {
         /* Update transmit ring FIFO tail pointer */
         __BUF_INCR( rb.tx_tail );
      }
      else
      {
         break;
      }
   }

   /* If there is no more data to send, disable the transmit
      interrupt - else enable it or keep it enabled */
   if( __BUF_IS_EMPTY( rb.tx_head, rb.tx_tail ) )
   {
      HAL_UART_ConfigInterrupt( ( UARTn_Type* )UART1, UARTn_INTCFG_THRE, DISABLE );
      // Reset Tx Interrupt state
      TxIntStat = RESET;
   }
   else
   {
      // Set Tx Interrupt state
      TxIntStat = SET;
      HAL_UART_ConfigInterrupt( ( UARTn_Type* )UART1, UARTn_INTCFG_THRE, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART receive function (ring buffer used)
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void IntReceive( void )
{
   uint8_t     tmpc;
   uint32_t    rLen;

   while( 1 )
   {
      // Call UART receive function in UART driver
      rLen = HAL_UART_Receive( ( UARTn_Type* )UART1, &tmpc, 1, NONE_BLOCKING );

      // If data received
      if( rLen )
      {
         /* Check if buffer is more space
          * If no more space, remaining character will be trimmed out
          */
         if( !__BUF_IS_FULL( rb.rx_head, rb.rx_tail ) )
         {
            rb.rx[rb.rx_head] = tmpc;
            __BUF_INCR( rb.rx_head );
         }
      }
      // no more data
      else
      {
         break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART Line Status Error
 * @param[in]     bLSErrType
 *                   UART Line Status Error Type
 * @return        None
 *//*-------------------------------------------------------------------------*/
void IntErr( uint8_t bLSErrType )
{
   uint8_t     test;

   // Loop forever
   while( 1 )
   {
      // For testing purpose
      test = bLSErrType;
      test = test;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART transmit function for interrupt mode (using ring buffers)
 * @param[in]     UARTx
 *                   Pointer to the target UART
 *                   -  UART1
 * @param[in]     txbuf
 *                   Pointer to Transmit buffer
 * @param[in]     buflen
 *                   Length of Transmit buffer
 * @return        Number of bytes actually sent to the ring buffer
 *//*-------------------------------------------------------------------------*/
uint32_t rbSend( UARTn_Type* UARTx, uint8_t* txbuf, uint8_t buflen )
{
   uint8_t*    data = ( uint8_t* ) txbuf;
   uint32_t    bytes = 0;

   /* Temporarily lock out UART transmit interrupts during this
      read so the UART transmit interrupt won't cause problems
      with the index values */
   HAL_UART_ConfigInterrupt( UARTx, UARTn_INTCFG_THRE, DISABLE );

   /* Loop until transmit run buffer is full or until n_bytes expires */
   while( ( buflen > 0 ) && ( !__BUF_IS_FULL( rb.tx_head, rb.tx_tail ) ) )
   {
      /* Write data from buffer into ring buffer */
      rb.tx[rb.tx_head] = *data;
      data++;

      /* Increment head pointer */
      __BUF_INCR( rb.tx_head );

      /* Increment data count and decrement buffer size count */
      bytes++;
      buflen--;
   }

   /*
    * Check if current Tx interrupt enable is reset,
    * that means the Tx interrupt must be re-enabled
    * due to call IntTransmit() function to trigger
    * this interrupt type
    */
   if( TxIntStat == RESET )
   {
      IntTransmit();
   }
   /*
    * Otherwise, re-enables Tx Interrupt
    */
   else
   {
      HAL_UART_ConfigInterrupt( UARTx, UARTn_INTCFG_THRE, ENABLE );
   }

   return bytes;
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART receive function for interrupt mode (using ring buffers)
 * @param[in]     UARTx
 *                   Pointer to the target UART
 *                   -  UART1
 * @param[out]    rxbuf
 *                   Pointer to Received buffer
 * @param[in]     buflen
 *                   Length of Received buffer
 * @return        Number of bytes actually read from the ring buffer
 *//*-------------------------------------------------------------------------*/
uint32_t rbReceive( UARTn_Type* UARTx, uint8_t* rxbuf, uint8_t buflen )
{
   uint8_t*    data = ( uint8_t* ) rxbuf;
   uint32_t    bytes = 0;

   /* Temporarily lock out UART receive interrupts during this
      read so the UART receive interrupt won't cause problems
      with the index values */
   HAL_UART_ConfigInterrupt( UARTx, UARTn_INTCFG_RBR, DISABLE );

   /* Loop until receive buffer ring is empty or until max_bytes expires */
   while( ( buflen > 0 ) && ( !( __BUF_IS_EMPTY( rb.rx_head, rb.rx_tail ) ) ) )
   {
      /* Read data from ring buffer into user buffer */
      *data = rb.rx[rb.rx_tail];
      data++;

      /* Update tail pointer */
      __BUF_INCR( rb.rx_tail );

      /* Increment data count and decrement buffer size count */
      bytes++;
      buflen--;
   }

   /* Re-enable UART interrupts */
   HAL_UART_ConfigInterrupt( UARTx, UARTn_INTCFG_RBR, ENABLE );

   return bytes;
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp, tmp1;

   /* Determine the interrupt source */
   intsrc = UART1->IIR;
   tmp = intsrc & UARTn_IIR_INTID_MASK;

   // Receive Line Status
   if( tmp == UARTn_IIR_INTID_RLS )
   {
      // Check line status
      tmp1 = UART1->LSR;
      // Mask out the Receive Ready and Transmit Holding empty status
      tmp1 &= ( UARTn_LSR_OE | UARTn_LSR_PE | UARTn_LSR_FE | UARTn_LSR_BI );
      // If any error exist
      if( tmp1 )
      {
         IntErr( tmp1 );
      }
   }

   // Receive Data Available or Character time-out
   if( tmp == UARTn_IIR_INTID_RDA )
   {
      IntReceive();
   }

   // Transmit Holding Empty
   if( tmp == UARTn_IIR_INTID_THRE )
   {
      IntTransmit();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   /*
    * Initialize UART1 pin connect
    */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF1 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 7, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, AFSRx_AF1 );

   // default: 38400-8-N-1
   HAL_UART_ConfigStructInit( &UARTn_Config );
   HAL_UART_Init( ( UARTn_Type* )UART1, &UARTn_Config );

   /* Enable UART Rx interrupt */
   HAL_UART_ConfigInterrupt( ( UARTn_Type* )UART1, UARTn_INTCFG_RBR, ENABLE );
   /* Enable UART line status interrupt */
   HAL_UART_ConfigInterrupt( ( UARTn_Type* )UART1, UARTn_INTCFG_RLS, ENABLE );

   TxIntStat = RESET;

   // Reset ring buf head and tail idx
   rb.rx_head = 0;
   rb.rx_tail = 0;
   rb.tx_head = 0;
   rb.tx_tail = 0;

   NVIC_SetPriority( UART1_IRQn, 3 );
   NVIC_EnableIRQ( UART1_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_UART1 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         UARTn_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UARTn_InterruptRun( void )
{
   uint32_t    idx, len;
   uint8_t     buffer[10];

   rbSend( ( UARTn_Type* )UART1, ( uint8_t* )"Interrupt test \n\r", 17 );

   /* Infinite loop */
   while( 1 )
   {
      len = 0;
      while( len == 0 )
      {
         len = rbReceive( ( UARTn_Type* )UART1, buffer, sizeof( buffer ) );
      }

      /* Got some data */
      idx = 0;
      while( idx < len )
      {
         if( buffer[idx] == 0x0d )
         {
            rbSend( ( UARTn_Type* )UART1, ( uint8_t* )"\n\r 123456789012334567890", ( 2 + 22 ) );
         }
         else
         {
            /* Echo it back */
            rbSend( ( UARTn_Type* )UART1, &buffer[idx], 1 );
            rbSend( ( UARTn_Type* )UART1, ( uint8_t* )"[1234567890123345678] \n\r", ( 2 + 22 ) );
         }
         idx++;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* UART1_Configure */
   UART1_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* UARTn_InterruptRun */
   UARTn_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

