/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void SysTick_ExceptionHandler_IT( void );
void WDT_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void WDT_Configure( void );
void WDT_ResetRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "WDT_Reset Example \n\r"
                        "  - MCU:        A31G112 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: WDT \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    msec;
uint8_t              wdtf_uf;    // underflow flag
uint8_t              wdtf_wmf;   // window match flag

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         SysTick_ExceptionHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SysTick_ExceptionHandler_IT( void )
{
   if( msec > 0 )
   {
      msec--;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_IRQHandler_IT( void )
{
   uint32_t    status;

   status = HAL_WDT_GetStatus();
   if( status & 0x01 )     // Match Interrupt
   {
      if( wdtf_uf == 0 )
      {
         wdtf_uf = 1;
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
      }
      else
      {
         wdtf_uf = 0;
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
      }
      HAL_WDT_ClearStatus( 1 );
   }

   if( status & 0x02 )     // Window Interrupt
   {
      if( wdtf_wmf == 0 )
      {
         wdtf_wmf = 1;
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
      }
      else
      {
         wdtf_wmf = 0;
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 1 ) );
      }
      HAL_WDT_ClearStatus( 2 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_Configure( void )
{
   // WDT configuration
   {
      // select peripheral clock: WDTRC
      HAL_SCU_Peripheral_ClockSelection( PPCLKSR_WDTCLK, WDTCLK_WDTRC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_ResetRun
 * @param         None
 * @return        None
 * @attention     IAR toolchain users should pay attention to the WDT Reset Time.
 *                If WDT Reset Time is too short, it is hard to debugging.
 *                So, we recomend to use the WDT Reset Time larger than 2 seconnds.
 *//*-------------------------------------------------------------------------*/
void WDT_ResetRun( void )
{
   WDT_CFG_Type      WDT_Config;

#if 1 // WDT_DR(0.5s) < WDTWDR(1s), clear in 900ms ==> Reset  (underflow)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000 / 2;     // 0.5s
         WDT_Config.wdtWTmrConst = 10000;        // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 900;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(0.5s) < WDTWDR(1s), clear in 400ms ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000 / 2;     // 0.5s
         WDT_Config.wdtWTmrConst = 10000;        // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 400;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) == WDTWDR(1s), clear in 900ms ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;    // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;      // 1s
         WDT_Config.wdtWTmrConst = 10000;     // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 900;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) == WDTWDR(1s), clear in 1.2s ==> Reset (underflow)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;    // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;      // 1s
         WDT_Config.wdtWTmrConst = 10000;     // 1s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 1200;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) > WDTWDR(0.5s), clear in 0.2  ==> Reset (window)

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;         // 1s
         WDT_Config.wdtWTmrConst = 10000 / 2;    // 0.5s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 200;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif

#if 0 // WDT_DR(1s) > WDTWDR(0.5s), clear in 0.8s   ==> NO reset 

   // WDT configuration
   {
      // wdt configuration
      {
         // init wdt
         WDT_Config.wdtResetEn = ENABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;       // WDTCLK = WDTCLKIN / PRS
         WDT_Config.wdtTmrConst = 10000;         // 1s
         WDT_Config.wdtWTmrConst = 10000 / 2;    // 0.5s
         HAL_WDT_Init( &WDT_Config );

         // enable interrupt
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_UNFIEN, ENABLE );
         HAL_WDT_ConfigInterrupt( WDT_INTCFG_WINMIEN, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( WDT_IRQn, 3 );
      NVIC_EnableIRQ( WDT_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_WDT );

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* Infinite loop */
   while( 1 )
   {
      msec = 800;
      while( msec );
      HAL_WDT_ReloadTimeCounter();
   }

#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* WDT_Configure */
   WDT_Configure();

   // configure SysTick to 1ms interrupt
   SysTick_Config( SystemCoreClock / 1000 );

   /* WDT_ResetRun */
   WDT_ResetRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

