/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void Buffer_Init( uint8_t* buffer, uint8_t type );
void AL5524_PORT_INIT( void );
void AL5524_LED_DISPLAY( uint32_t number );
uint16_t adcaverage( uint16_t* val );
void SysTick_ExceptionHandler_IT( void );
void TIMER10_IRQHandler_IT( void );
void I2C0_IRQHandler_IT( void );
void GPIO_Configure( void );
void TIMER10_Configure( void );
void ADC_Configure( void );
void I2C0_Configure( void );
void DEMO_LEDRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

#define AL5524_ADDR        (0x40 >> 1)
/** Own Slave address in Slave I2C device */
#define I2CDEV_S_ADDR      (0xC0 >> 1)

/** Max buffer length */
#define BUFFER_SIZE        5

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

volatile uint32_t       msec;

Bool                    complete;
Bool                    isMasterMode = TRUE;
/** These global variables below used in interrupt mode - Slave device ----------------*/
uint8_t                 Master_Buf[BUFFER_SIZE];

TIMER1n_PWMCFG_Type     TIMER1n_Config;
volatile uint32_t       count;
volatile uint32_t       flag;

const char              LEDSEG[17] =
{
   0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xD8, 0x80, 0x98, 0x88, 0x83, 0xC6, 0xA1, 0x86, 0x8E, 0x7F
   //0    1      2      3       4       5       6      7      8      9      A       b      c      d       E      F      ' '
};

uint16_t                adcval[8];

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Initializes buffer
 * @param[in]     buffer
 *                buffer to initialize
 * @param[in]     type:
 *                   -  0:    Initialize buffer with 0
 *                   -  1:    Initialize buffer with increment value from 0
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Buffer_Init( uint8_t* buffer, uint8_t type )
{
   uint8_t     i;

   if( type )
   {
      for( i = 0; i < BUFFER_SIZE; i++ )
      {
         buffer[i] = i;
      }
   }
   else
   {
      for( i = 0; i < BUFFER_SIZE; i++ )
      {
         buffer[i] = 0;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         AL5524_PORT_INIT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void AL5524_PORT_INIT( void )
{
   I2Cn_M_SETUP_Type    transferMCfg;
   uint8_t              Master_Buf[BUFFER_SIZE];

   /* Pin Confige - Output */
   complete = FALSE;
   Buffer_Init( Master_Buf, 0 );
   Master_Buf[0] = 0x06; //address
   Master_Buf[1] = 0x00; //data
   transferMCfg.sl_addr7bit = AL5524_ADDR;
   transferMCfg.tx_data = Master_Buf;
   transferMCfg.tx_length = 2;
   HAL_I2C_Master_Transmit( ( I2Cn_Type* )I2C0, &transferMCfg, I2Cn_TRANSFER_INTERRUPT );
   // HAL_I2C_Master_Transmit((I2Cn_Type*)I2C1, &transferMCfg, I2Cn_TRANSFER_INTERRUPT);
   while( !complete );
}

/*-------------------------------------------------------------------------*//**
 * @brief         AL5524_LED_DISPLAY
 * @param[in]     number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void AL5524_LED_DISPLAY( uint32_t number )
{
   I2Cn_M_SETUP_Type    transferMCfg;
   uint8_t              Master_Buf[BUFFER_SIZE];

   /* Pin Confige - Output */
   complete = FALSE;
   Buffer_Init( Master_Buf, 0 );
   Master_Buf[0] = 0x02; //address
   Master_Buf[1] = LEDSEG[number]; //data
   transferMCfg.sl_addr7bit = AL5524_ADDR;
   transferMCfg.tx_data = Master_Buf;
   transferMCfg.tx_length = 2;
   HAL_I2C_Master_Transmit( ( I2Cn_Type* )I2C0, &transferMCfg, I2Cn_TRANSFER_INTERRUPT );
   // HAL_I2C_Master_Transmit((I2Cn_Type*)I2C1, &transferMCfg, I2Cn_TRANSFER_INTERRUPT);
   while( !complete );
}

/*-------------------------------------------------------------------------*//**
 * @brief         adcaverage
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
uint16_t adcaverage( uint16_t* val )
{
   uint32_t    i;
   uint32_t    adcavg;

   adcavg = 0;
   for( i = 0; i < 8; i++ )
   {
      adcavg += val[i];
   }
   adcavg >>= 3;

   return adcavg;
}

/*-------------------------------------------------------------------------*//**
 * @brief         SysTick_ExceptionHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SysTick_ExceptionHandler_IT( void )
{
   if( msec > 0 )
   {
      msec--;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_IRQHandler_IT( void )
{
   uint32_t    status;

   status = T1nMaInt_GetFg( TIMER10 );
   if( status == 1 )
   {
      T1nMaInt_ClrFg( TIMER10 );
      count++;
      if( count >= 1000 )
      {
         count = 0;
         if( flag == 0 )
         {
            flag = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         else
         {
            flag = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void I2C0_IRQHandler_IT( void )
{
   if( isMasterMode ) // Master Mode
   {
      HAL_I2C_Master_IRQHandler_IT( ( I2Cn_Type* )I2C0 );
      if( HAL_I2C_Master_GetState( ( I2Cn_Type* )I2C0 ) )
      {
         complete = TRUE;
      }
   }
   else// Slave Mode
   {
      HAL_I2C_Slave_IRQHandler_IT( ( I2Cn_Type* )I2C0 );
      if( HAL_I2C_Slave_GetState( ( I2Cn_Type* )I2C0 ) )
      {
         complete = TRUE;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // T10O pin config set if need
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 6, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 6, AFSRx_AF1 );

   // Test Pin setting PB0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );

   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );

   // I2C0 PD0:SCL0, PD1:SDA0
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 0, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 0, PUPDx_Disable );

   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 1, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 1, PUPDx_Disable );

   // I2C1   PA1:SCL1, PA0:SDA1
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 0, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 0, PUPDx_Disable );

   HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 1, PUPDx_Disable );

   //AN2 setting PA2
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 2, AFSRx_AF3 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, ALTERN_FUNC );

   //Input Pin setting PF6
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 6, INPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PF, 6, PUPDx_Disable );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_Configure( void )
{
   TIMER1n_Config.CkSel = TIMER1n_PCLK;               // PCLK = 10MHz
   TIMER1n_Config.Prescaler = 10;                     // fTIMER = PCLK / 10 = 1MHz
   TIMER1n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
   TIMER1n_Config.BDR = ( 300 );                      // Duty(B) = BDR / fTIMER =  300us
   TIMER1n_Config.StartLevel = TIMER1n_START_HIGH;    // TIMER1n_START_LOW, TIMER1n_START_HIGH

   HAL_TIMER1n_Init( ( TIMER1n_Type* )TIMER10, TIMER1n_PWM_MODE, &TIMER1n_Config );
   HAL_TIMER1n_ConfigInterrupt( ( TIMER1n_Type* )TIMER10, TIMER1n_INTCFG_MIE, ENABLE );

   NVIC_SetPriority( TIMER10_IRQn, 3 );
   /* Enable Interrupt for TIMERn channel */
   NVIC_EnableIRQ( TIMER10_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_TIMER10 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_Configure( void )
{
   ADC_CFG_Type      ADC_Config;

   // ADC configuration
   {
      // adc configuration
      {
         ADC_Config.RefSel = ADC_REF_VDD;    // 0:ADC_REF_VDD ,1:ADC_REF_AVREF
         ADC_Config.TrgSel = ADC_TRIGGER_DISABLE;
         ADC_Config.InClkDiv = 4;            // SystemPeriClock/3M = 3.3
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         I2C0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void I2C0_Configure( void )
{
   // Initialize Slave I2C peripheral
   HAL_I2C_Init( ( I2Cn_Type* )I2C0, 100000 );

   HAL_INT_EInt_MaskDisable( MSK_I2C0 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEMO_LEDRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEMO_LEDRun( void )
{
   uint32_t       tmp = 0;
   uint32_t       adcavg, i;

   while( 1 )
   {
      msec = 100;
      while( msec );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

      msec = 100;
      while( msec );
      HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );

      HAL_ADC_ChannelSel( ADC, 2 ); // select ch

      for( i = 0; i < 8; i++ )
      {
         HAL_ADC_ClearStatus( ADC );
         HAL_ADC_Start( ADC ); // start
         while( ( HAL_ADC_GetStatus( ADC ) & ( ADC_STAT_END ) ) != ( ADC_STAT_END ) ) {}
         adcval[i] = HAL_ADC_GetData( ADC );
      }

      adcavg = adcaverage( adcval );

      isMasterMode = TRUE;
      AL5524_LED_DISPLAY( ( adcavg >> 8 ) );

      while( PF->INDR_b.INDR6 == 0 )
      {
         HAL_TIMER1n_Cmd( ( TIMER1n_Type* )TIMER10, ENABLE );   // timer start & clear

         msec = 150;
         while( msec );

         HAL_TIMER1n_Cmd( ( TIMER1n_Type* )TIMER10, DISABLE );   // timer start & clear
      }

      if( tmp & 0x1 )
      {
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 1 ) );
      }
      else
      {
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
      }
      if( ++tmp == 17 )
      {
         tmp = 0;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER10_Configure */
   TIMER10_Configure();

   /* ADC Configure */
   ADC_Configure();

   /* I2C0_Configure */
   I2C0_Configure();

   /* AL5524_PORT_INIT */
   AL5524_PORT_INIT();

   // configure SysTick to 1ms interrupt
   SysTick_Config( SystemCoreClock / 1000 );

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DEMO_LEDRun */
   DEMO_LEDRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

