/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER30_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER30_Configure( void );
void TIMER3n_CaptureRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER3n_Capture Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:             TIMER30 \n\r"
                        "Capture Signal Generation Pin: PE0 \n\r"
                        "Timer Capture Input Pin:       PF7(T30CAP) \n\r"
                        "Prerequisite:                  Connect PE0 and PF7(T30CAP) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    fflag = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER30_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER30_IRQHandler_IT( void )
{
   uint32_t    status;

   // get interrupt flag
   status = TIMER30->INTFLAG_b.T3nCIFLAG;

   // if Capture Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      HAL_TIMER3n_ClearStatus_IT( ( TIMER3n_Type* )TIMER30, TIMER3n_INTFLAG_T3nCIFLAG_Msk );

      // clear counter and prescaler
      TIMER3n_ClearCounter( TIMER30 );

      // update fflag
      if( fflag == 0 )
      {
         fflag = 1;
      }
      else if( fflag == 1 )
      {
         fflag = 2;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // capture signal generation pin configurtion
   {
      // print
      _DBG( "PE0 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PE0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER30_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER30_Configure( void )
{
   TIMER3n_CFG_Type     TIMER3n_Config;

   // TIMER30 configuration
   {
      // timer capture input pin configuration
      {
         // print
         _DBG( "PF7 was configured as a T30CAP. \n\r\n\r" );

         // configure PF7 as a T30CAP
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 7, AFSRx_AF2 );   // PF7            0: ----        1: ----        2: T30CAP      3: ----        4: ----
      }

      // timer capture mode configuration
      {
         // select peripheral clock: PCLK
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_T30CLK, T30CLK_PCLK );

         // init timer
         TIMER3n_Config.T3nMS = TIMER3n_CR_T3nMS_CaptureMode;
         TIMER3n_Config.T3nCLK = TIMER3n_CR_T3nCLK_IntPrescaledClock;   // PCLK = 40MHz
         TIMER3n_Config.T3nECE = TIMER3n_CR_T3nECE_FallingEdge;         // dummy parameter
         TIMER3n_Config.T3nCPOL = TIMER3n_CR_T3nCPOL_RisingEdge;
         TIMER3n_Config.PDR = 1000;                                     // Period  = PDR / fTIMER = 1000us
         TIMER3n_Config.ADR = 0;                                        // Duty(A) = ADR / fTIMER =    0us  // dummy parameter
         TIMER3n_Config.BDR = 0;                                        // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
         TIMER3n_Config.CDR = 0;                                        // Duty(C) = CDR / fTIMER =    0us  // dummy parameter
         TIMER3n_Config.Prescaler = 40 - 1;                             // fTIMER = PCLK / 40 = 1MHz
         HAL_TIMER3n_Init( ( TIMER3n_Type* )TIMER30, &TIMER3n_Config );
         HAL_TIMER3n_ConfigInterrupt( ( TIMER3n_Type* )TIMER30, ENABLE, TIMER3n_INT_CAPEN );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER30_IRQn, 3 );
      NVIC_EnableIRQ( TIMER30_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER30 );

      // enable timer (clear & start counter)
      TIMER3n_EnableTimer( TIMER30 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER3n_CaptureRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER3n_CaptureRun( void )
{
   uint32_t    i;
   uint32_t    count;

   // generate capture signal
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );     // high
   for( i = 0; i < 1000; i++ );                    // 1.22ms
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );   // low
   for( i = 0; i < 1000; i++ );                    // 200us
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );     // high

   /* Infinite loop */
   while( 1 )
   {
      switch( fflag )
      {
         case 2:
            // updae fflag
            fflag = 0;

            // get capture data
            count = TIMER3n_GetCaptureData( TIMER30 );

            // print capture data
            _DBG( "Capture CNT = " );
            _DBD32( count );
            _DBG( "\n\r" );     // Capture data : 400
            if( count == 400 )
            {
               _DBG( "Capture CNT is Ok. \n\r" );
            }
            else
            {
               _DBG( "Capture CNT is not Ok. \n\r" );
            }
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER30_Configure */
   TIMER30_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER3n_CaptureRun */
   TIMER3n_CaptureRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

