/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void WDT_Configure( void );
void WDT_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "WDT_Polling Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: WDT \n\r"
                        "************************************************\n\r\n\r";

uint8_t     wdtf_uf;    // underflow flag
uint8_t     wdtf_wmf;   // window match flag

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_Configure( void )
{
   WDT_CFG_Type      WDT_Config;

   // WDT configuration
   {
      // wdt configuration
      {
         // select peripheral clock: WDTRC
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_WDTCLK, WDTCLK_WDTRC );

         // init wdt
         WDT_Config.wdtResetEn = DISABLE;
         WDT_Config.wdtClkDiv = WDT_DIV_4;      // WDTCLK = WDTCLKIN / PRS = 40k/4 = 10k = 0.1ms
         WDT_Config.wdtTmrConst = 25000;        // 2.5s
         WDT_Config.wdtWTmrConst = 5000;        // 0.5s
         HAL_WDT_Init( &WDT_Config );
      }

      // start & reload wdt
      HAL_WDT_Start( ENABLE );
      HAL_WDT_ReloadTimeCounter();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WDT_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WDT_PollingRun( void )
{
   uint32_t    status;

   /* Infinite loop */
   while( 1 )
   {
      do
      {
         status = ( HAL_WDT_GetStatus() & 0x03 );
      } while( status == 0 );

      if( status & 0x01 )
      {
         HAL_WDT_ReloadTimeCounter();

         if( wdtf_uf == 0 )
         {
            wdtf_uf = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         else
         {
            wdtf_uf = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         HAL_WDT_ClearStatus( 1 );
      }

      if( status & 0x02 )
      {

         if( wdtf_wmf == 0 )
         {
            wdtf_wmf = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
         }
         else
         {
            wdtf_wmf = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 1 ) );
         }
         HAL_WDT_ClearStatus( 2 );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* WDT_Configure */
   WDT_Configure();

   /* WDT_PollingRun */
   WDT_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

