/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void WT_Configure( void );
void WT_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "WT_Polling Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: WT \n\r"
                        "************************************************\n\r\n\r";

uint8_t     wtif;  // flag

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         WT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WT_Configure( void )
{
   WT_CFG_Type    WT_Config;

   // WT configuration
   {
      // XSOSC SXIN/SXOUT pin configuration
      {
         HAL_SCU_SubXtal_PinConfig();
         HAL_SCU_ClockSource_Enable( CLKSRCR_XSOSCEN, HIRCSEL_HIRC1 );
      }

      // wt configuration
      {
         // select peripheral clock: XSOSC(32.768kHz)
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_WTCLK, WTCLK_XSOSC );

         // init wt
         WT_Config.wtClkDiv = WT_DIV_2_13;      // 32768/(2^13) = 4Hz  ==> 250ms
         HAL_WT_Init( &WT_Config );
      }

      // start wt (clear the counter and divider & start counter)
      HAL_WT_Start( ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         WT_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void WT_PollingRun( void )
{
   uint32_t    status;

   /* Infinite loop */
   while( 1 )
   {
      do
      {
         status = ( HAL_WT_GetStatus() & 0x02 );
      } while( status == 0 );

      if( status & 0x02 )
      {
         if( wtif == 0 )
         {
            wtif = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         else
         {
            wtif = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         HAL_WT_ClearStatus();
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* WT_Configure */
   WT_Configure();

   /* WT_PollingRun */
   WT_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

